// |$Id: bext.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bext.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: extract seismic traces based on header            |
   |                                                            |
   |  extraction by:  spname  rstnam    ofrn                    |
   |                  shot    receiver  field rec               |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         int *ivalue, char extsw[], char cvalue[], char infile[],
         char ofile[], char lstfile[],char pid[]);

      void outlst(int ivalue, char extsw[], char cvalue[],char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1;
      char              pid[5]="bext",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      int               ivalue;     //for field record number
      int		jrec2;	    //output trace counter
      int		itest;	    //test for comparison
      int		flag;	    //flag 1=write trace, a match
      char		extsw[2];   //extraction switch
      char              cvalue[5];  //for shot or rec. name
      char              ctest[5];   //test for comparison

//...get parameters
      if(getparm(argc,argv,&ivalue,extsw,cvalue,
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(ivalue,extsw,cvalue,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %6.5f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
/*        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
*/
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop
	jrec2=0;	//initialize output trace counter
      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...extract traces that match pattern
	flag=0;
	switch(extsw[0])
	{
	  case 's':
	    { strncpy(ctest,hd.spname,4);
	      ctest[4]='\0';
	      if(strncmp(ctest,cvalue,4)==0) flag=1; 
	    break;}
	  case 'r':
	    { strncpy(ctest,hd.rstnam,4);
	      ctest[4]='\0';
	    if(strncmp(ctest,cvalue,4)==0) flag=1; 
	    break;}
	  case 'f':
	   {itest=(int) hd.ofrn;
	    if(itest==ivalue)  flag=1;
	    break;}
	} //endswitch
//...output trace
	if(flag==1)
	{
        if(c_bsegout(jrec2,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }
	jrec2=jrec2+1;  	//increment trace counter
	} //endif

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      return(0);

quit_it:
        puts(" ");

/*        free(s2);
exit_s1:
*/
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
      return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *ivalue, char extsw[], char cvalue[], char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char		buf[81];
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"bext:  Extracts traces based on header       ");
      sprintf(msg2,"values chosen by user.                       ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile   extsw    value\n",pid);
      puts(" ");
      puts("     infile   =  input file name");
      puts("     extsw    = extraction switch (1 char)");
      puts("             s= shot name  ");
      puts("             r= receiver name  ");
      puts("             f= field record number ");
      puts("     value    = shot or rec name (4 char) ");
      puts("                       or ");
      puts("                field record number (int) ");
      puts(" ");
      puts("  WARNING: leading blanks are important   ");
      puts("  (enclose 4 char string in quotes if on command line)");
      puts(" ");
      puts("   Use bdump program to find names of shots or receivers");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       {extsw[1]='\0';
       strncpy(extsw,argv[2],1);}
       else
      {
      puts(" Enter extsw extraction switch (1 char)");
      puts("             s= shot name  ");
      puts("             r= receiver name   ");
      puts("             f= field record ");
       scanf("%1c",extsw);
	fgets(buf,80,stdin);  //flush buffer for next scanf
       extsw[1]='\0';
      }

//...input value
      if(extsw[0]=='s' || extsw[0]=='r')
	{ *ivalue=0;
	  if(argc>3)
	  {strncpy(cvalue,argv[3],4);
	   cvalue[4]='\0';}
	  else
	  {puts(" Enter extraction value (4char)");
           puts("  WARNING: leading blanks are important   ");
	   puts("  (no quotes here, just space bar if needed)");
	   scanf("%4c",cvalue);
	   cvalue[4]='\0';}
	}
	else
	{strncpy(cvalue,"    ",4);
	  cvalue[4]='\0';
	  if(argc>3)
	  {*ivalue=atoi(argv[3]);}
	  else
	  {puts(" Enter extraction value (int)");
	  scanf("%d",ivalue);}
	} //endif

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int ivalue, char extsw[], char cvalue[],char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     extsw= %s ",extsw);
      fprintf(h3,"\n     ivalue= %5.5d ",ivalue);
      fprintf(h3,"\n     cvalue= %s ",cvalue);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
