// |$Id: genbrot.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: genbrot.c                                   |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: generate a batch file for rotation                |
   |          of seismic traces using BROT and                  |
   |          HODO.LST                                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   |------------------------------------------------------------|
*/
#include <stdio.h>
# include "../include/c_bsegy.h"
# include "../include/sub4.h"


//...parameter definitions

//...function prototypes


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1;

      int               jfirst,
                        jlast,
                        nfiles,
                        j,
                        k;
      char              help[3]="-h";

//      char              ofile[13]="gobrot.bat",
      char              ofile[13]="gobrot",
                        infile[13],
                        postfx[2],
                        prfx4[13],
			pid[5]="gbrt",
                        prfx[13];
//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);
      puts("INTERACTIVE");
      puts("Generates bash script to run program, brot,    ");
      puts("on a collection of files.  Program brot requires file    ");
      puts("hodo.lst (contains down-hole tool orientation)   ");
      puts("The hodogram analysis resulting in hodo.lst is usually  ");
      puts("done by running programs genbhod and bhod.  ");
      puts("BROT rotates the horizontal traces into a standard orientation.");
      puts("  ");
      puts("  ");
      return(0);
     } //endif

//display copyright
      cr_labl(pid);
      
      if( (h1=fopen(ofile,"w"))==NULL )
      {
       printf("\n ABORT  !! Error opening file %s",ofile);
       goto exit1;
      }

//...input parameters
      puts("  ");
      puts("Enter alpha prefix (char) of *.seg data to be rotated");
      puts("  ");
      puts("EXAMPLE: if enter 1, then files 1001.seg to 1010.seg  ");
      puts("                     would be processed if sequence ");
      puts("                     numbers 1 and 10 entered next ");
      puts("  ");
      puts("  ");

      scanf("%s",prfx);
        for(j=0;j<13;j++)
       {
        if(prfx[j]=='\0')
	{
	 postfx[0]=prfx[j-1];
	 break;
	}  //endif
       } //endfor
        postfx[1]='\0';
	fprintf(stdout,"\n ...%s \n",postfx);
        k=0;
        for(j=0;j<4;j++)
        {
         if(prfx[j]=='\0') goto escp1;
         prfx4[j]=prfx[j];
         k=k+1;
        }  //endfor
escp1:
        prfx4[k]=0;

      puts(" Enter first file number to process");
      scanf("%d",&jfirst);

      puts(" Enter last file number to process");
      scanf("%d",&jlast);

      nfiles=jlast-jfirst+1;

//...loop and write commands to *.bat file
      for(j=0;j<nfiles;j++)
     {
      sprintf(infile,"%s%3.3d.seg",prfx,j+1);
//      fprintf(h1," call BROT %s 2 3 1\n",infile);
      fprintf(h1,"  brot %s 2 3 1\n",infile);
       if(k!=1)
      {
//       sprintf(infile,"BROT%s.seg",prfx4);
       sprintf(infile,"brot%s.seg",prfx4);
//       fprintf(h1," move %s BROT%s%3.3d.seg\n",infile,postfx,j+1);
       fprintf(h1," mv %s brot%s%3.3d.seg\n",infile,postfx,j+1);
      }  //endif

     }  //endif

//      puts(" Output in file===>gobrot.bat");
      puts(" Output in file===>gobrot");

      fclose(h1);
exit1:
      return(0);

     } //endmain
