c |$Id: bcad.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bcad.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | bcad converts x,y,z coord. to autocad *.DXF file           |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2016 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     FINDXYZ Compute x,y,z coordinates of source and recvrs.|
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767,mdim=1000)
c
      real*4            s1(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
c
      real*4            ssx(mdim),ssy(mdim),ssz(mdim)
      real*4            rrx(mdim),rry(mdim),rrz(mdim)
      character*4       rstn(mdim)
      character*4       sstn(mdim)
c
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bcad'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +isw1,ilabel,txtsiz)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...clear output file...
      write(outfil,'(a,a,''.dxf'')') aplc(1:4),infil(1:4)
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,isw1,ilabel,txtsiz)
c
      icall1=0
      icall2=0
c
c...data loop=======================================....
      fxmax=-1.E+10
      fxmin=+1.E+10
      fymax=fxmax
      fymin=fxmin
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...get geometry
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c...save geometry
      ssx(jrec)=sx
      ssy(jrec)=sy
      ssz(jrec)=sz-sd
      rrx(jrec)=rx
      rry(jrec)=ry
      rrz(jrec)=rz
      rstn(jrec)=rstnam
      sstn(jrec)=spname
c
c...search max and min x,y
      if(fxmax.lt.rx) fxmax=rx
      if(fxmax.lt.sx) fxmax=sx
      if(fymax.lt.ry) fymax=ry
      if(fymax.lt.sy) fymax=sy
c
      if(fxmin.gt.rx) fxmin=rx
      if(fxmin.gt.sx) fxmin=sx
      if(fymin.gt.ry) fymin=ry
      if(fymin.gt.sy) fymin=sy
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  100 continue
c
c...open output file *.dxf format
      open(unit=4,file=outfil,access='sequential')
c
c...write header with max and min limits
      if(isw1.eq.1) then
      write(4,900) fxmin,fymin,fxmax,fymax
  900 format('  0'/'SECTION'/'  2'/'HEADER'/'  9'/'$LIMMIN'/
     1' 10'/f16.6/' 20'/f16.6/'  9'/'$LIMMAX'/
     2' 10'/f16.6/' 20'/f16.6/'  0'/'ENDSEC'/
     3'  0'/'SECTION'/'  2'/'ENTITIES')
      else
      write(4,903)
  903 format('  0'/'SECTION'/'  2'/'ENTITIES')
      endif
c
      write(io11,800) fxmin,fymin,fxmax,fymax
  800 format(5x,'MINIMUM X,Y'/5x,2(f16.6,1x)/
     15x,'MAXIMUM X,Y'/5x,2(f16.6,1x)//)
c
c...output origin (0,0,0)
      origin=0.0
      write(4,901) origin,origin,origin
      txtsiz10=10.0*txtsiz
      write(4,908) origin,origin,origin,txtsiz10,lineid
  908 format('  0'/'TEXT'/'  8'/'RED'/
     1' 10'/f16.6/' 20'/f16.6/' 30'/f16.6/' 40'/f10.3/
     2'  1'/' Line: ',a/'  7'/'ROMANS')
c
c...output shot location
      write(4,901) ssx(1),ssy(1),ssz(1)
  901 format('  0'/'POINT'/'  8'/'0'/
     1' 10'/f16.6/' 20'/f16.6/' 30'/f16.6)
c
      write(io11,801) ssx(1),ssy(1),ssz(1),spname
  801 format(5x,'SHOT LOCATION: sx,sy,sz'/5x,3(f16.6,1x),5x,'SP',a4)
      write(io11,803)
  803 format(//5x,'RECEIVER LOCATIONS: rx,ry,rz')
c
c...output receiver locations
      do 200 jrec=1,ntrace
      write(4,901) rrx(jrec),rry(jrec),rrz(jrec)
c
      write(io11,802) rrx(jrec),rry(jrec),rrz(jrec),rstn(jrec)
  802 format(5x,3(f16.6,1x),5x,'VP',a4)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
  200 continue
c
c...output labels
      if(ilabel.eq.1) then
      do 55 jrec=1,ntrace
      rstnam=rstn(jrec)
      write(4,905) rrx(jrec),rry(jrec),rrz(jrec),txtsiz,rstnam(2:4)
  905 format('  0'/'TEXT'/'  8'/'RED'/
     1' 10'/f16.6/' 20'/f16.6/' 30'/f16.6/' 40'/f10.3/
     2'  1'/'VP',a/'  7'/'ROMANS')
   55 continue
      write(4,907) ssx(1),ssy(1),ssz(1),txtsiz,spname(2:4)
  907 format('  0'/'TEXT'/'  8'/'RED'/
     1' 10'/f16.6/' 20'/f16.6/' 30'/f16.6/' 40'/f10.3/
     2'  1'/'SP',a/'  7'/'ROMANS')
      endif
c
c...end section
      write(4,906)
  906 format('  0'/'ENDSEC'/'  0'/'EOF')
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(4)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +isw1,ilabel,txtsiz)
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2016  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bcad: Read headers and create a *.dxf    |'
      write(*,*) '| file for autocad or microstation         |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bcad       infile   isw1   ilabel    txtsiz'
      write(*,*) ' '
      write(*,*) 'infile  =*.seg input file name '
      write(*,*) 'isw1    =switch to control limits '
      write(*,*) '       0=no limits header '
      write(*,*) '       1=limits based on min and max values '
      write(*,*) 'ilabel 0=no printing of point labels  '
      write(*,*) '       1=print labels '
      write(*,*) 'txtsiz  =size of text in coord. units (float) '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  enter: isw1'
      write(*,*) '       0=supress limits header '
      write(*,*) '       1=include max and min for limits header '
      read(*,*) isw1
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') isw1
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*) 'ilabel 0=no printing of point labels  '
      write(*,*) '       1=print labels '
      read(*,*) ilabel
      else
      call getarg(3,arg3)
      read(arg3,'(i5)') ilabel
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)'txtsiz  =size of text in coord. units (float) '
      read(*,*) txtsiz
      else
      call getarg(4,arg4)
      read(arg4,'(f10.3)') txtsiz
      endif
c
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,isw1,ilabel,txtsiz)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bcad  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   isw1= '',i5)') isw1
      if(isw1.eq.0) then
      write(io11,*) '     Supress limits header'
      else
      write(io11,*) '     Include limits header'
      endif
      write(io11,'(''ilabel = '',i5)') ilabel
      write(io11,'(''txtsiz = '',f7.2)') txtsiz
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
