c |$Id: bdump.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bdump.f                                      |
c | DIRECTORY: bsu-2.0.3/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Performs a partial header dump to a listing file.          |
c |                                                            |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     FINDXYZ Finds x,y,z coordinates source and receivers   |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      real*4            s1(ndim)
      character*40      infil , aplc, outlst
      character*31      fil2c
      character*31      blankl
      character*1       blank
c
      common /header/ hd
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bdmp'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +isupr)                      
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,outlst)
c
c
      icall1=0
      icall2=0
c
c...data loop=======================================....
c
c...initialize shot switch and trace counter
      lcount=60
      isw1=-99
      ntr = 0
c
c...pad label 
      write(blank,'('' '')')
      do 13 jc=1,40
      if(infil(jc:jc).eq.blank) then
      ncharf=jc-1
      go to 14
      endif
   13 continue
   14 continue
      do 15 jc=1,31
   15 blankl(jc:jc)=blank
      lpad=(31-ncharf)/2
      write(fil2c,'(a,a)') blankl(1:lpad),infil(1:ncharf)
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      dt=fsamin
c
c...read location headers and compute
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c
c...new page
      if(ofrn.ne.isw1 .or. lcount.gt.55 ) then
      isw1=ofrn
      if(isupr.ne.0 .or. jrec.eq.1) then
      if(jrec.eq.1) write(io11,900) char(32),fil2c(1:31)
      if(jrec.ne.1) write(io11,900) char(12),fil2c(1:31)
      write(io11,901) length,sz,dt,sd,ictime,utimes,flc,sx
      write(io11,905) fhc,sy,lineid,tyear,tday,thour,tmin,shtazi,
     +shtver,charge
      write(io11,902)
      write(io11,903)
      lcount=20
      else
      lcount=10
      endif
      endif
c
c...write header and computed info
c...first break pick in sec.(mst=microsec, int*4)
      pick=float(mst)/1.E+06
      write(io11,904) jrec,ofrn,spname,rstnam,range,rz,rx,ry,
     +nvs,pick,gidb,geoazi,geover
c
      lcount=lcount+1
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
      write(*,'('' '')')
      write(6,240) ntr,outlst
240   format(' Traces Processed = ',i5/
     +5x,/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
c
c...format block
  900 format(a/21x,'|-------------------------------|'/
     +21x,'|   PARTIAL SEGY HEADER DUMP    |'/
     +21X,'|                               |'/
     +21X,'|',A,'|'/
     +21X,'|                               |'/
     +21X,'|-------------------------------|')
c
  901 format(/1x,83(1h-)/3x,'Length = ',i5,' samples',13X,'|',
     +2x,'Shot Elevation = ',f10.1/
     +3x,'Sample Interval = ',f10.5,' sec.',2X,'|',
     +2x,'Shot Depth = ',f10.1/
     +3x,'Delay Time = ',i4,' msec.',12x,'|',
     +2x,'Up Hole Time = ',i5,' msec'/
     +3x,'Low Cut Filter  = ',i4,' Hz.',9X,'|',
     +2x,'Shot X-COORD = ',f10.2)
  905 format(
     +3x,'High Cut Filter = ',i4,' Hz.',9X,'|',
     +2x,'Shot Y-COORD = ',f10.2/
     +3x,'Line ID: ',a,22x,'|',
     +2x,'Shot Date (year.moday) = ',i4,'.',i4.4/
     +3x,'Shot Orientation:     ',4x,9x,'|',
     +2x,'Shot Time (hr:min)   = ',i2.2,':',i2.2/
     +3x,'Azimuth=',i3,' Deg.',2x,'Vertical=',i3,' Deg.|',
     +2x,'Charge Size (grams)= ',i5)
c
  902 format(1x,83(1h-))
c
  903 format(' TRACE|','SHOT|',' STATION |',' OFFSET|',
     +'        RECEIVER          |','VERT|','1STBRK|','K-GAIN|AZI|VER|'/
     +'   #  |','REC.|','SHOT  REC|','       |',
     +'  ELEV.  X-COORD   Y-COORD|','FOLD|','(SEC.)|',' (dB) |   |   |'/
     +1X,'-----|----|---------|-------|----------------------------|',
     +'--|------|------|---|---|')
c
  904 format(1x,i4,1x,'|',i4,'|',a4,1x,a4,'|',f7.2,'|',
     +f8.2,f10.2,f10.2,
     +'|',i2,'|',f6.4,'|',2x,i3,1x,'|',i3,'|',i3,'|')
c
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +isupr)                     
c
      character*40      infil
      character*40      arg1,arg2
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bdump:  Partial seismic trace header     |'
      write(*,*) '| dump (shows locations, picks, etc.)      |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bdump infile iskip '
      write(*,*)' '
      write(*,*)'infile:    =name of input file '
      write(*,*)'iskip:     =0 supress shot header display after 1st '
      write(*,*)'           =1 display all shot headers '
      write(*,*)' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c...test supress shot header display after first
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter option:  0=supress shot headers after 1st'
      write(*,*)'                 1=show all shot headers'
      read(*,*) isupr
      else
      call getarg(2,arg2)
      read(arg2,'(i5)') isupr
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,outlst)
      character*40      outlst
c
c...open output listing file...
      io11=11
      write(outlst,'(''bdump.lst'')') 
      open(io11,file=outlst,access='sequential')
      return
      end
