c |$Id: bftr.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bftr.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | bftr applies filters to a bsegy data set. Filter may be    |
c | specified by NAMELIST file, or *.SEG data set              |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     CONV2   Convolution subroutine with user zero reference|
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim),fil(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, filef
c
      namelist /filter/ npf,nzph,fil
c
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      iunit3=3
      icall1=0
      icall2=0
      icall3=0
c
c...define application prefix
      write(aplc,'(''bftr'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +iswf,filef)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
      if(iswf.eq.0) then
      call CHKTRC(iunit3,icall3,npts3,s1,fsamin3,filef,
     +bar3,invbar3,ibar3,ntrace3,ndim)
c
      if(ntrace.ne.ntrace3) then
      write(*,*) ' ABORT--Number of filter traces .NE. infile'
      stop
      endif
      endif
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iswf,filef)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      icall3=0
      ntr=0
c
c...input filter by namelist if iswf=1
c...NOTE: if you forget the &end at the end of the file, you will get
c         a CORE DUMP, so be sure it's there!
      if(iswf.eq.1) then
      open(4,file=filef,access='sequential')
      read(4,filter)
      if(npf.gt.ndim) then
      write(*,*) ' ABORT--filter exceeds max dimensions'
      stop
      endif
      if(nzph.lt.0 .or. nzph.gt.npf) then
      write(*,*) '  ABORT--zero reference sample out of bounds'
      stop
      endif
c
c...echo check of namelist
      write(io11,*) '&filter'
      write(io11,'(''  npf='',i5.5,'', nzph='',i5.5,'','')') npf,nzph
      write(io11,*) 'fil= '
      write(io11,'(8(f9.4,'',''))') (fil(j),j=1,npf)
      write(io11,*) '&end'
      endif
c
c...data loop=======================================....

c
      do 100 jrec=1,ntrace
c
c...read filter if iswf=0
      if(iswf.eq.0) then
      CALL BSEGIN(iunit3,ndim,
     +icall3,npf,fil,fsamn3,filef,jrec,iexit)
      nzph=1
c
      if(fsamn3.ne.fsamin) then
      write(*,'('' WARNING--UNEQUAL SAMPLE INTERVALS'')')
      write(*,'('' infile fsamin= '',f10.5,'' filef fsamin='',f10.5)')
     +fsamin,fsamn3
      write(io11,'('' WARNING--UNEQUAL SAMPLE INTERVALS'')')
      write(io11,'('' infile fsamin= '',f10.5,'' filef fsamin='',
     +f10.5)') fsamin,fsamn3
      endif
      endif
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1

c
c...apply filter to input data
      call conv2(npts,s1,npf,fil,s2,nzph)
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  100 continue
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(iunit3)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +iswf,filef)
c
      character*40      infil,filef
      character*40      arg1,arg2
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bftr: Filter seismic traces with either  |'
      write(*,*) '| namelist file, or other *.seg file.      |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bftr   infil   iswf    filef '
      write(*,*) ' '
      write(*,*) 'infil:    =input file to be filtered '
      write(*,*) 'iswf:     =filter source switch  '
      write(*,*) '         0=filters in *.SEG data set  '
      write(*,*) '          (one filter trace for each trace in infil)'
      write(*,*) '         1=single filter specified in namelist file '
      write(*,*) '     '
      write(*,*) 'filef:    =name of filter data set '
      write(*,*) ' '
      write(*,*) '------Namelist Definitions------- '
      write(*,*) '&FILTER   npf=number of points in filter '
      write(*,*) '          nzph=sample for zero reference '
      write(*,*) '          fil=f1,f2,f3,f4,....'
      write(*,*) '              (values of filter samples) '
      write(*,*) '&end '
      write(*,*) ' '
      write(*,*) 'NOTE: if you get a core dump, you may have  '
      write(*,*) '      forgotten the &end at the end of the file'
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...read input parameters
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  0=filters in *.SEG file'
      write(*,*) '  1=filter input by namelist file'
      write(*,*) '  enter choice of filter input'
      read(*,*) iswf
      else
      call getarg(2,arg2)
      read(arg2,'(i1)') iswf
      endif
c
      if(nargsx.lt.(4+karg)) then
      if(iswf.eq.1) then
      write(*,*) ' enter filter NAMELIST file name '
      else
      write(*,*) ' enter filters *.SEG file name'
      endif
      read(*,'(a)') filef
      else
      call getarg(3,filef)
      endif
c
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iswf,filef)
      character*4       aplc
      character*40      outlst,outfil,infil,filef
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c

      write(io11,'('' Program bftr'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   iswf= '',i5)') iswf
      write(io11,'(''   filef= '',a)') filef
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
