c |$Id: bmrg.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bmrg.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Merge a number of shot records into a single file          |
c | (can select specific traces from the shot records)         |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
      real*4            s1(ndim)
      character*40      infil , outfil, aplc, outlst, pfix
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bmrg'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,
     +pfix,iffile,ilfile,ifinc,iftrc,iltrc,nfiles,nchar)
c...name the first input file              
      kfile=iffile
      if(ilfile.lt.10000) then
      write(infil,'(a,i4.4,''.seg'')') pfix(1:nchar),kfile
      endif
      if(ilfile.lt.1000) then
      write(infil,'(a,i3.3,''.seg'')') pfix(1:nchar),kfile
      endif
      if(ilfile.lt.100) then
      write(infil,'(a,i2.2,''.seg'')') pfix(1:nchar),kfile
      endif
      if(ilfile.lt.10) then
      write(infil,'(a,i1.1,''.seg'')') pfix(1:nchar),kfile
      endif
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...special case, nfiles, not number of traces for progress bar
      bar=float(nfiles)/50.
c     bar=bar/float(ifinc)
      ibar=int(bar+0.5)
      invbar=int(1./bar+0.5)
c
c...name outputfile
      write(outfil,'(a,''.seg'')') aplc(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,aplc,outlst,
     +pfix,iffile,ilfile,ifinc,iftrc,iltrc,nfiles)
c
c...clear output file...
      write(*,'(a)') outfil
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
      ntr = 0
      kfile=iffile-ifinc
      krec=0
c
      do 550 jfile=1,nfiles
c
      kfile=kfile+ifinc
      if(kfile.gt.ilfile) go to 550
      close(iunit1)
      icall1=0
      iexit=0
c
      if(ilfile.lt.10000) then
      write(infil,'(a,i4.4,''.seg'')') pfix(1:nchar),kfile
      endif
      if(ilfile.lt.1000) then
      write(infil,'(a,i3.3,''.seg'')') pfix(1:nchar),kfile
      endif
      if(ilfile.lt.100) then
      write(infil,'(a,i2.2,''.seg'')') pfix(1:nchar),kfile
      endif
      if(ilfile.lt.10) then
      write(infil,'(a,i1.1,''.seg'')') pfix(1:nchar),kfile
      endif
      write(io11,'(5x,a)') infil
c
c...BEGIN LOOP====================================....
      do 500 jrec=iftrc,iltrc
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      krec=krec+1
c
      if(iexit.eq.1) go to 600
      ntr = ntr + 1
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,krec)
c
  500 continue
c
      call pltbar(bar,invbar,ibar,jfile,nfiles)
  550 continue
c
c...END LOOP========================================....
600   continue
c
      write(*,'('' '')')
      write(6,240) ntr,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,
     +pfix,iffile,ilfile,ifinc,iftrc,iltrc,nfiles,nchar)
c
      character*40      pfix
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bmrg: Merge traces from many files into  |'
      write(*,*) '|       a single output file.              |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bmrg pfix iffile ilfile ifinc iftrc iltrc  '
      write(*,*)' '
      write(*,*)'pfix:  =prefix for input file names '
      write(*,*)' NOTE:pfix length is no. of invarient charcters'
      write(*,*)'  Ex. If file names run s001 to s090 then pfix=s0'
      write(*,*)'  Ex. If file names run s001 to s132 then pfix=s'
      write(*,*)'iffile =number of first file (suffix) '
      write(*,*)'  EXAMPLE:  if file=s001.seg, iffile=001 '
      write(*,*)' '
      write(*,*)'ilfile =number of last file (suffix) '
      write(*,*)'  EXAMPLE:  if file=s092.seg, ilfile=092 '
      write(*,*)' '
      write(*,*)'ifinc  =increment for file number (suffix) '
      write(*,*)'iftrc  =first trace each file '
      write(*,*)'iltrc  =last trace each file  '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: input file prefix '
      read(*,'(a13)') pfix
      else
      call getarg(1,pfix)
      endif
      call namechk(12,pfix)
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: first file number '
      read(*,*) iffile
      else
      call getarg(2,arg2)
      read(arg2,'(i4)') iffile
      endif
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  enter: last file number '
      read(*,*) ilfile
      else
      call getarg(3,arg3)
      read(arg3,'(i4)') ilfile
      endif
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  enter: file number increment '
      read(*,*) ifinc
      else
      call getarg(4,arg4)
      read(arg4,'(i4)') ifinc
      endif
c
      nchar=0
      do 5 j=1,8
      if(pfix(j:j).eq.' ') then
      nchar=j-1
      go to 6
      endif
    5 continue
    6 continue
c     write(*,*) nchar,pfix(1:nchar)
c
      if(nargsx.lt.(6+karg)) then
      write(*,*)'  enter: first trace to include in each file '
      read(*,*) iftrc
      else
      call getarg(5,arg5)
      read(arg5,'(i4)') iftrc
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,*)'  enter: last trace to include in each file '
      read(*,*) iltrc
      else
      call getarg(6,arg6)
      read(arg6,'(i4)') iltrc
      endif
c
      nfiles=ilfile-iffile+1
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,aplc,outlst,
     +pfix,iffile,ilfile,ifinc,iftrc,iltrc,nfiles)
      character*4       aplc
      character*40      outlst
      character*40      pfix
c
c...open output listing file...
      io11=11
      write(outlst,'(a,''.lst'')') aplc
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
c     write(*,'(''  Number of files='',i5)') nfiles
      write(io11,'(''  Number of files='',i5)') nfiles
      write(io11,'(''  Parameters:      '')')
      write(io11,'(''     prefix=  '',a)') pfix
      write(io11,'(''     first file #= '',i5)') iffile
      write(io11,'(''     last file #= '',i5)') ilfile
      write(io11,'(''     file # increment= '',i5)') ifinc
      write(io11,'(''     first trace (each file)= '',i5)') iftrc
      write(io11,'(''     last trace (each file)= '',i5)') iltrc
      write(io11,'(''  '')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

