c |$Id: bshf.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bshf.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Static shift traces according to header time picks or      |
c | the contents of a pick file:  (trace #,   shift seconds)   |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     VSHFT   Interpolates sample at time between samples    |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, picfil
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
      io12=12
      write(picfil,'(''      '')')
c
c...define application prefix
      write(aplc,'(''bshf'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,io12,
     +ipic,ishf,tshift,picfil)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,ipic,ishf,tshift,picfil)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
c...zero output trace
      do 60 jt=1,npts
   60 s2(jt)=0.
c
      icall1=0
      icall2=0
      ntr=0
      jrec=0
c
c...data loop=======================================....
      write(io11,*) ' TRACE       PICK        SHIFT'
c
      do 100 jrecx=1,ntrace
c
c...read pick if from file
      if(ipic.eq.1) then
      read(12,*,end=600) jrec,pick
      else
      jrec=jrec+1
      endif
c
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1

      if(ipic.ne.1) then
      pick=float(mst)/1.0E+06
      endif
c
c...output trace pick aligned...
      write(io11,'(2x,i5.5,2x,f10.4,$)') jrec,pick
      if(ishf.eq.0) then
      pick=-pick
      endif
c
      dt=pick-tshift
      write(io11,'(2x,f10.4)') dt
      call VSHFT(npts,s1,s2,dt,fsamin)
c
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
      goto 601
600   write(io11,*) 'Pick file ended too soon'
      write(*,*) 'Pick file ended too soon'
601   continue
      write(*,*) ' '
      write(6,240) ntr,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(io12)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,io12,
     +ipic,ishf,tshift,picfil)
c
      character*40      infil, picfil
      character*40      arg1,arg2,arg3,arg4
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bshf: Static shift seismic traces.       |'
      write(*,*) '|       (shifts from headers or file).     |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'BSHF infile ipic ishf tshift picfil'
      write(*,*)' '
      write(*,*)' infile:   =input file name '
      write(*,*)' ipic:    1=static shift data by picks pick file'
      write(*,*)'          0=static shift data by picks in headers'
      write(*,*)' ishf:    0=add static shifts '
      write(*,*)'          1=subtract static shifts '
      write(*,*)' tshift:   =bulk static to add to picks '
      write(*,*)' picfil:  =file name with picks (for ipic=0)'
      write(*,*)' '
      stop
      endif
      endif

      if(nargsx.ge.(3+karg)) then
      call getarg(2,arg2)
      read(arg2,'(i1)') ipic
      else
      write(*,*)'  0=use pick in header   1=use pic file'
      read(*,*) ipic
      endif
c
      if(ipic.eq.1) then
        if(nargsx.ge.(2+karg)) then
        call getarg(1,infil)
        else
        write(*,*)'  enter: name of input file'
        read(*,'(a13)') infil
        endif
      call namechk(12,infil)

        if(nargsx.ge.(6+karg)) then
        call getarg(5,picfil)
        else
        write(*,*)'  enter: name of pic file'
        read(*,'(a13)') picfil
        endif
      call namechk(12,picfil)
c
c...open pick file
        open(io12,file=picfil,access='sequential')
      else
        if(nargsx.ge.(2+karg)) then
        call getarg(1,infil)
        else
        write(*,*)'  enter: name of input file'
        read(*,'(a13)') infil
        endif
      call namechk(12,infil)
      endif
c
c...read parameters
      if(nargsx.ge.(4+karg)) then
      call getarg(3,arg3)
      read(arg3,'(i1)') ishf
      else
      write(*,'(''    0= add statics in pick file'')')
      write(*,'(''    1= subtract statics in pick file'')')
      read(*,*) ishf
      endif
      if(nargsx.ge.(5+karg)) then
      call getarg(4,arg4)
      read(arg4,'(f10.4)') tshift
      else
      write(*,'(''    enter any bulk shift in seconds'')')
      read(*,*) tshift
      endif

      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,ipic,ishf,tshift,picfil)
      character*4       aplc
      character*40      outlst,outfil,infil,picfil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bshf  '')')
      write(io11,'('' Input File: '',a)') infil
      write(11,'('' Pick File: '',a)') picfil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   ipic= '',i5)')    ipic        
      write(io11,'(''   ishf= '',i5)')    ishf        
      write(io11,'('' tshift= '',f10.2)') tshift
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
      return
      end
