c |$Id: bshp.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bshp.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |              Least Squares Shaping Filter Program          |
c |              (data file#1 * filter) = data file #2         |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     SHAPE   Waveshaping filter by E. A. Robinson           |
c |             (Calls cross, eureka, dot, fold)               |
c |     FOLD    Convolves two signals, E. A. Robinson          |
c |     EUREKA  Solves normal equations by Levinson recursion  |
c |     DOT     Computes inner (dot) product of two vectors    |
c |     CROSS   Cross correlates two signals                   |
c |     TLIST   Time domain printer plot of signal             |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      external shape
      parameter (mdimy=32767,ndim=2*mdimy)
      real*4            s1(mdimy),s2(mdimy)
      character*40      infil , outfil, aplc, outlst
      character*40      infil2, infil3
c
      character*70 title
      real*4 b(ndim),a(ndim),d(ndim),c(ndim),space(3*ndim)
      real*4 sumfil(ndim)
ccc---These can be used to look at frequency domain view of operator
ccc   (search for more "ccc" comments to see other code to uncomment)
ccc   complex x(ndim)
ccc   real*4 xr(ndim),xi(ndim),amp(ndim),phz(ndim)
c
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      iunit3=3
      iunit4=4
      icall1=0
      icall2=0
      icall3=0
      icall4=0
c
c...define application prefix
      write(aplc,'(''bshp'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +infil2,iswopt,iswch,tmin1,tmax1,npf,sf,infil3)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,mdimy)
      call CHKTRC(iunit2,icall2,npts2,s1,fsamin2,infil2,
     +bar2,invbar2,ibar2,ntrace2,mdimy)
      if(iswch.eq.0) then
        call CHKTRC(iunit3,icall3,npts3,s1,fsamin3,infil3,
     +  bar3,invbar3,ibar3,ntrace3,mdimy)
c
c...check sample rates
         if(fsamin.ne.fsamin3) then
         write(*,*)' ABORT!!!--Sample rates unequal #1 and #3'
         stop
         endif
      endif
c
c...check time gate
      istart=int(tmin1/fsamin) + 1
      istop=int(tmax1/fsamin) +1
      if(istop.gt.npts) then
      istop=npts
      tmax1=float(istop-1)*fsamin
      write(*,'(''  warning--tmax reset'',f10.4)') tmax1
      endif
c
      if(istart.lt.1) istart=1
      ilengt=istop-istart+1
      write(*,'(''  gate length='',i5,'' points'')') ilengt
c
c...check sample rates
      if(fsamin.ne.fsamin2) then
      write(*,*) fsamin,fsamin2
      write(*,*)' ABORT!!!--Sample rates unequal #1 and #2'
      stop
      endif
c
c...name outputfile
      if(iswch.eq.0) then
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil3(1:4)
      else
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      endif
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,ilengt,
     +infil,infil2,iswopt,iswch,tmin1,tmax1,npf,sf,infil3)
c
c...clear output file...
      open(11,file=outfil,access='direct',recl=240)
      close(11,status='delete')
c
c...initialize sumfil
      do 12 j=1,ndim
   12 sumfil(j)=0.0
c
      icall1=0
      icall2=0
      icall3=0
      icall4=0
      ntr=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
      jrec2=jrec
c
c...input a trace
      call BSEGIN(iunit1,mdimy,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...input a trace
      call BSEGIN(iunit2,mdimy,
     +icall2,npts2,s2,fsamin2,infil2,jrec2,iexit2)
c
c
c...apply gate
      lb=0
      do 201 jt=istart,istop
      lb=lb+1
      b(lb)=s1(jt)
      d(lb)=s2(jt)
  201 continue
      ld=lb
      la=npf
      lc=la+lb-1
      call shape(lb,b,ld,d,la,a,lc,c,ase,space,sf)
      write(io11,'(''  ase='',e14.7)') ase
c
      if(iswopt.eq.0) then
c...>>>>sum filters
c...sum filters for an average
      do 30 jt=1,la
      sumfil(jt)=sumfil(jt)+a(jt)
   30 continue
c...>>>>>apply on fly
      else
      write(title,'('' least squares filter #'',i5)') ntr
      call tlist(la,a,title)
c
c...choose data set to apply to
c----------------------------------------
      if(iswch.eq.0) then
c
      CALL BSEGIN(iunit3,mdimy,
     +icall3,npts3,s1,fsamin3,infil3,jrec,iexit3)
c
      lc=la+npts3-1
      call fold(npts3,s1,la,a,lc,c)
c
c...output trace
      CALL BSEGOUT(iunit4,
     +icall4,npts3,c,fsamin3,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c--------------------
      else
c...or     use the existing s1 trace
      lc=la+npts-1
      call fold(npts,s1,la,a,lc,c)
c
c...output trace
      CALL BSEGOUT(iunit4,
     +icall4,npts,c,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
      endif
c--------------------------------
      endif
c
c
c...END LOOP========================================....
  100 continue
c
c
c=================profile mode only=====
      if(iswopt.eq.0) then
c
c...average filters
      do 25 j=1,la
      sumfil(j)=sumfil(j)/float(ntr)
   25 continue
c
      write(title,'('' least squares filter '')')
c
      call tlist(la,sumfil,title)
c
ccc   call nrad2(la,npr2)
ccc   do 35 j=1,la
ccc   x(j)=cmplx(sumfil(j),0.0)
ccc35 continue
ccc   lap1=la+1
ccc   do 40 j=lap1,npr2
ccc   x(j)=cmplx(0.0,0.0)
ccc40 continue
ccc
ccc   sign=-1.0
ccc   call nlogn(npr2,x,sign)
ccc
ccc   ilin=1
ccc   tzero=0
ccc   samin=fsamin
ccc   call flist(npr2,x,title,samin,tzero,xr,xi,amp,phz,ilin)
c
      endif
c==========================
c
      if(iswopt.eq.1) then
      write(6,240) ntr,outfil,outlst
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      endif
c
      close(iunit1)
      close(iunit2)
      close(iunit3)
      close(iunit4)
c
c....profile mode only////////////////////////////////
      if(iswopt.eq.0) then
c
      ntr = 0
      jrec = 0
c
      icall1=0
      icall3=0
      icall4=0
c
c...data loop=======================================....
c
      do 200 jrec=1,ntrace
      if(iswch.eq.1) then
c
c...input a trace
      CALL BSEGIN(iunit1,mdimy,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      lc=la+npts-1
      call fold(npts,s1,la,sumfil,lc,c)
c
c...output filtered trace
      CALL BSEGOUT(iunit4,
     +icall4,npts,c,fsamin,outfil,jrec)

      else
c
      CALL BSEGIN(iunit3,mdimy,
     +icall3,npts3,s1,fsamin3,infil3,jrec,iexit3)
c
      lc=la+npts3-1
      call fold(npts3,s1,la,sumfil,lc,c)
c
c...output filtered trace
      CALL BSEGOUT(iunit4,
     +icall4,npts3,c,fsamin3,outfil,jrec)
c
      endif
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  200 continue
c
      write(*,'('' '')')
      write(6,240) ntr,outfil,outlst
      endif
c//////////////////////////////////////////////////////
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(iunit3)
      close(iunit4)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +infil2,iswopt,iswch,tmin1,tmax1,npf,sf,infil3)
c
      character*40      infil
      character*40      arg1,arg3,arg4,arg5,arg6,arg7,arg8
      character*40      infil2, infil3
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
c c---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bshp: Wiener Least Squares Shaping Filter|'
      write(*,*) '| (apply to data or alternative file)      |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bshp infil  infil2 iswopt iswch tmin tmax npf sf infil3 '
      write(*,*)' '
      write(*,*)'infil :   =name of input file #1 '
      write(*,*)'           (file1*filter=file2)  '
      write(*,*)'infil2:   =name of input file #2 '
      write(*,*)'iswopt:  0=profile mode '
      write(*,*)'         1=trace mode '
      write(*,*)'iswch:   1= (file#1*filter)=output '
      write(*,*)'         0= (file#3*filter)=output '
      write(*,*)'tmin:     =start time of design gate in seconds '
      write(*,*)'tmax:     =end time of design gate in seconds '
      write(*,*)'npf:      =number of point in filter '
      write(*,*)'sf:       =stability factor (.001 typical)  '
      write(*,*)'infil3:   =name of input file #3 (iswch=0 only) '
      write(*,*)' '
      stop
      endif
      endif
c
      write(infil3,'(1h )')
c
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file#1'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: name of input file#2'
      read(*,'(a13)') infil2
      else
      call getarg(2,infil2)
      endif
      call namechk(12,infil2)
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  Select Option'
      write(*,*)'  0=profile mode'
      write(*,*)'  1=trace mode'
      read(*,*) iswopt
      else
      call getarg(3,arg3)
      read(arg3,'(i1)') iswopt
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  Select Option'
      write(*,*)'  1 = apply match filter(s) to input file#1'
      write(*,*)'  0 = apply match filter(s) to another data set'
      read(*,*) iswch
      else
      call getarg(4,arg4)
      read(arg4,'(i1)') iswch
      endif
c
      if(iswch.eq.0) then
        if(nargsx.lt.(10+karg)) then
        write(*,*)'  enter: name of other file to apply filter to'
        read(*,'(a)') infil3
        else
        call getarg(9,infil3)
        endif
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'(''  enter: start time of design gate (sec.)'')')
      read(*,*) tmin1
      else
      call getarg(5,arg5)
      read(arg5,'(f10.4)') tmin1
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,'(''  enter: end time of design gate (sec.)'')')
      read(*,*) tmax1
      else
      call getarg(6,arg6)
      read(arg6,'(f10.4)') tmax1
      endif
c
      if(nargsx.lt.(8+karg)) then
      write(*,'(''  enter: number of points in filter'')')
      read(*,*) npf
      else
      call getarg(7,arg7)
      read(arg7,'(i5)') npf
      endif
c
      if(nargsx.lt.(9+karg)) then
      write(*,'(''  enter: Stability Factor 0<=SF<=1 '')')
      write(*,*)' '
      write(*,*)' The larger SF, the more stable, but less effective'
      read(*,*) sf 
      else
      call getarg(8,arg8)
      read(arg8,'(f10.4)') sf 
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,ilengt,
     +infil,infil2,iswopt,iswch,tmin1,tmax1,npf,sf,infil3)
      character*4       aplc
      character*40      outlst,outfil,infil
      character*40      infil2, infil3
c
c...open output listing file...
      io11=10
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bshp'')')
      write(io11,'('' Input File#1: '',a)') infil
      write(io11,'('' Input File#2: '',a)') infil2
      write(io11,'('' Input File#3: '',a)') infil3
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
c
      if(iswopt.eq.0) then
      write(io11,'(''     PROFILE MODE (iswopt=0) '')')
      else
      write(io11,'(''     TRACE MODE   (iswopt=1) '')')
      endif
      if(iswch.eq.1) then
      write(io11,'(''     Apply Filter to File #1 (iswch=1)'')')
      else
      write(io11,'(''     Apply Filter to File #3 (iswch=0)'')')
      endif
      write(io11,'(''     Tmin= '',f10.4)') tmin1
      write(io11,'(''     Tmax= '',f10.4)') tmax1
      write(io11,'(''     Gate length='',i5,'' points'')') ilengt
      write(io11,'(''     Number of points in filter= '',i5)') npf
      write(io11,'(''     Stability Factor = '',f10.4)') sf 
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
