c |$Id: genbhod.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: genbhod.f                                    |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |   This is an interactive program that generates the scripts|
c |  for a batch processing which will lead to determination   |
c |  of down-hole tool orientation by PCA analysis.  The       |
c |  preparatory step is a wave field enhancement of the SH-   |
c |  wave by nulling out Rayleigh and P-waves, based on        |
c |  signals obtained from a reference phone.                  |
c |                                                            |
c |    There are many assumptions here.  First, it is assumed  |
c |  that a stationary 3-C reference phone is at the surface,  |
c |  and that a source is able to generate SH-waves with the   |
c |  main polarization direction being generally East-West     |
c |  (note the reference to 270 and 90 degree azimuth).        |
c |  Note that the bowspring indicates the direction of the    |
c |  R-component down-hole, but that is only for my tool.  Your|
c |  tool may be different.  Take it appart to find out.       |
c |  The UP/DOWN switch part provides flexibility in handling  |
c |  the azimuth which you start to acquire data.  That is,    |
c |  if the first file is for 270 degree blow, it is one way   |
c |  (+1), if the first file is for a 90 degree blow, it is the|
c |  other way (-1).                                           |
c |    Further, it is assumed that the field records have 6    |
c |  channels.  Channels are defined as follows:               |
c |     CH     Description                                     |
c |     1      Down-hole vertical   (up)                       |
c |     2      Down-hole radial (aligned with bowspring)       |
c |     3      Down-hole transverse (orthogonal to 2)          |
c |     4      Reference vertical   (up)                       |
c |     5      Reference radial     (0 degree azimuth)         |
c |     6      Reference transverse (270 degree azimuth)       |
c |                                                            |
c |    This is not really a problem if you acquire data with a |
c |  different source polarization, but you must work out your |
c |  own sign conventions and prompts.  The programs bscl.f    |
c |  and bsum.f are used to scale and subtract (hence null out)|
c |  the Rayleigh and P-wave signals.  This conditioning helps |
c |  with the hodogram analysis by bhod.f, Principal Component |
c |  Analysis (PCA).  It is also assumed that you observed the |
c |  tool orientation on exiting the borehole with the down-   |
c |  hole phone, and can use that to resolve the 180 degree    |
c |  ambiguity inherent in the PCA analysis (Michaels, 2001)   |
c |                                                            |
c |  Michaels, P., 2001, "Use of Principal Component Analysis  |
c |   to Determine Down-Hole Tool Orientation and Enhance      |
c |   SH-Waves", Journ. Environmental and Engineering          |
c |   Geophysics, Vol. 6, No. 4, December, p175-183.           |
c |                                                            |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c
      character*8  prfx
c
      call GETPRM(nargsx)      
      write(*,*) 'enter 1char_ALPHA PREFIX '
      read(*,'(a)') prfx
      do 5, j=1,8
      if(prfx(j:j).eq.' ') then
      klast=j-1
      go to 6
      endif
    5 continue
    6 continue
      write(*,*) ' enter FIRST FILE NUMBER (<=3digits)'
      write(*,*) ' for which source polarization is 270 deg.'
      read(*,*) ifrst

      write(*,*) ' enter LAST FILE NUMBER (<=3digits)'
      write(*,*) ' for which source polarization is 270 deg.'
      read(*,*) ilast

      write(*,*) ' enter UP/DOWN SWITCH'
      write(*,*) '   -1= 90 Azimuth File Number 1 LESS than 270 Az'
      write(*,*) '   +1= 90 Azimuth File Number 1 MORE than 270 Az'
      read(*,*) iupdn
c
      write(*,*) ' enter azimuth of bowspring(R-comp)'
      read(*,*) azctl

c.============================DOWNHOLE 
      open(unit=2,access='sequential',file='gobhodo')
      open(unit=3,access='sequential',file='gorunbhod')
      nlines=(ilast-ifrst)+1
      icount=ifrst

      do 10 j=1,nlines,2
      kcount=icount+isign(1,iupdn)
      write(2,'(''bscl '',a,i3.3,''.seg 1 1 3'')') prfx(1:klast),kcount
      write(2,'(''bscl '',a,i3.3,''.seg 1 1 3'')') prfx(1:klast),icount
      write(2,100) prfx(1:klast), kcount, prfx(1:klast), icount
  100 format(' bsum bscl',a,i3.3,'.seg  bscl',a,i3.3,'.seg -1.0')
      write(2,101)   kcount,icount
c
  101 format(' mv bsumbscl.seg h',i3.3,i3.3,'.seg')
      write(3,'(''bhod h'',i3.3,i3.3,
     +''.seg 2 3 50 90.0'',f6.1,'' +90.0'')')
     +kcount,icount,azctl
      icount=icount+2
   10 continue
      write(*,*) ' OUTPUT====> Downhole: gobhodo'
      close(2)
      close(3)
c.============================REFERENCE
      open(unit=2,access='sequential',file='gobhodoR')
      open(unit=3,access='sequential',file='gorunbhodR')
      nlines=(ilast-ifrst)+1
      icount=ifrst

      do 20 j=1,nlines,2
      kcount=icount+isign(1,iupdn)
      write(2,'(''bscl '',a,i3.3,''.seg 4 1 3'')') prfx(1:klast),kcount
      write(2,'(''bscl '',a,i3.3,''.seg 4 1 3'')') prfx(1:klast),icount
      write(2,200) prfx(1:klast), kcount, prfx(1:klast), icount
  200 format(' bsum bscl',a,i3.3,'.seg  bscl',a,i3.3,'.seg -1.0')
      write(2,201)   kcount,icount
c
  201 format(' mv bsumbscl.seg r',i3.3,i3.3,'.seg')
      write(3,'(''bhod r'',i3.3,i3.3,
     +''.seg 5 6 50 90.0'',f6.1,'' -90.0'')')
     +kcount,icount,azctl
      icount=icount+2
   20 continue
      write(*,*) ' OUTPUT====> Reference: gobhodoR'
      close(2)
      close(3)
c.============================
      write(*,*) ' OUTPUT====> Downhole: gorunbhod'
      write(*,*) ' OUTPUT====> Reference: gorunbhodR'
      write(*,*) ' ----------------------------------------'
      write(*,*) 'REMEMBER to change permissions on the    '
      write(*,*) 'above files to execute. '
      write(*,*) ' ----------------------------------------'
      write(*,*) ' IF examining the Down-hole Phone        '
      write(*,*) '                                         '
      write(*,*) ' 1. Run gobhodo in directory with 6 chan '
      write(*,*) ' records (3 down, 3 reference phones)    '
      write(*,*) '                                         '
      write(*,*) ' 2. Run gorunbhod in directory with files'
      write(*,*) ' that are named hxxxyyy.seg              '
      write(*,*) '                                         '
      write(*,*) ' ----------------------------------------'
      write(*,*) ' IF examining the Reference Phone        '
      write(*,*) '                                         '
      write(*,*) ' 1. Run gobhodoR in the directory with   '
      write(*,*) ' the 6 channel records.                  '
      write(*,*) '                                         '
      write(*,*) ' 2. Run gorunbhodR in the directory with '
      write(*,*) ' files that are named rxxxyyy.seg        '
      write(*,*) '                                         '
      write(*,*) ' ----------------------------------------'
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx)      
c
      character*40      arg1
c
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
      write(*,*) ' '
      write(*,*) 'WARNING: !! '
      write(*,*) 'See Source Code, genbhod.f, or BSU documentation '
      write(*,*) '(man pages and BSU user Guide)                    '
      write(*,*) 'before you use this program. It is hardwired for'
      write(*,*) 'a specific type of acquisition.'
      write(*,*) ' '
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:        GENBHOD              |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| There are NO COMMAND LINE ARGUMENTS for  |'
      write(*,*) '| this program, it is interactive.         |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      endif
      return
      end
