c |$Id: top2nez.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: top2nez.f                                    |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | top2nez.f  Converts a Topcon Total Station survey file     |
c |            into *.nez file (Northing, Easting, Z-coord)    |
c |  (ie. puts everything on a single record per survey point) |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c
      character*40      infil, outfil
      character*12      snum, x, y, z, label
c
c...get file name
      call GETPRM(nargsx,infil)
c
      do 10, j=1,40
      if(infil(j:j).eq.' ') then
      jcnt=j-1
      go to 20
      endif
10    continue
20    write(outfil,'(a,''.nez'')') infil(1:jcnt)
      open(unit=1,file=infil,access='sequential',status='old')
      open(unit=2,file=outfil,access='sequential')
      close(2,status='delete')
      open(unit=2,file=outfil,access='sequential')
c
c...Start read loop----------------------                        
100   read (1,900,err=600,end=600) snum
      read (1,900,err=600,end=600) x
      read (1,900,err=600,end=600) y
      read (1,900,err=600,end=600) z
      read (1,900,err=600,end=600) label
      write(2,901) snum,x,y,z,label
      goto 100
600   continue
      close(2)
      close(1)
      write(*,'('' Output File= '',a)') outfil
c
c...FORMAT BLOCK
900   format (a)
901   format (5a12)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil)
c
      character*40      infil
      character*40      arg1
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|-------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN       |'
      write(*,*) '| ONLINE HELP:                              |'
      write(*,*) '|-------------------------------------------|'
      write(*,*) '| top2nez:  Converts raw Topcon survey file |'
      write(*,*) '|to NEZ format:required by topcon or topcon2|'
      write(*,*) '|-------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'top2nez    infile         '
      write(*,*) ' '
      write(*,*) 'infile  =input file name '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      return
      end
