c |$Id: topcon.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $        |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: topcon.f                                     |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |              Scans topcon files *.nez format and creates   |
c |              a *.xyz file for program bhed.f               |
c |              Scans bison file header for gain, date, etc.  |
c |------------------------------------------------------------|
c | input:[point#,northing(y-coord),easting(x-coord),elev,label]
c | *.nez input format: 5A12                                   |
c |                                                            |
c | output:[channel picktime spn sx sy sz vpn rx ry rz gain ]  |
c | (Note: pick times are set to zero)                         |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |  maxch= maximum number of channels per shot record         |
c |  CAUTION:ixvr variable, line 305 depends on geophone wiring|
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |                                                            |
c --------------------------------------------------------------
c
      parameter (ndim=32767, maxch=48)
      character*40      infil1,infil2,ofile
      character*12      point,ycoord,xcoord,elev,label
      character*2       SPL,VPL,LML,BML,BPL
      character*1       blank,dot
      integer*2         vp(ndim),chan(ndim)
      real*4            rx(ndim),ry(ndim),rz(ndim)
      real*4            sx,sy,sz,pick(ndim)
c
      character*80 line
      character*4  linenm
      integer*2         gidba(maxch),nvsa(maxch)
c
      write(blank,'(1h )')
      write(dot,'(1h.)')
      write(SPL,'(2hSP)')
      write(VPL,'(2hVP)')
      write(BML,'(2hBM)')
      write(BPL,'(2hBP)')
      write(LML,'(2hLM )')
c
c...get parameters
      call GETPRM(nargsx,infil1,infil2,linenm,fshdp,ispn,
     +nchanl,ivp1,ivp48,irec,eshift,isaz,isvr,irazi,itazi,islop)
c
      open(unit=1,file=infil1,access='sequential')
      do 2 j=1,40
      if(infil2(j:j).eq.blank .or. infil2(j:j).eq.dot) then
      jcnt=j-1
      go to 3
      endif
    2 continue
    3 continue
      write(ofile,'(a,''.xyz'')') infil2(1:jcnt)
c
c
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
c...Bison file data extraction
c...count traces...
      jrec = 0
      ntr = 0
      jchac=0
      icall=0
      iunit1=2
      write(*,5)
  5   format(' Reading Bison Header....')
 10   continue
      call readrc(iunit1,icall,infil2,jrec,line,nchar,iexit)
      if(iexit.eq.1) go to 20
      ntr = ntr + 1
c
c...capture selected variables
c...data sample format code
      if(ntr.eq.3) then
      read(line(1:4),'(i4)') idate
      read(line(5:6),'(i2)') iyear
      read(line(7:8),'(i2)') ihour
      read(line(9:10),'(i2)') imin
      endif
c
      if(ntr.eq.8) then
      if(line(2:2).eq.'W') idsfc=3
      if(line(2:2).eq.'L') idsfc=2
      if(line(2:2).eq.'F') idsfc=1
      write(*,'(''  Data Sample Format Code='',i1)') idsfc
      endif
c...number of channels
      if(ntr.eq.15) then
      read(line(1:4),'(i4)') nchan
      write(*,'(''  Number of Channels='',i5)') nchan
      endif
c...number of samples
      if(ntr.eq.17) then
      read(line(1:6),'(i6)') npts
      write(*,'(''  Number of Samples='',i5)') npts
      endif
c...sample interval
      if(ntr.eq.18) then
      read(line(1:4),'(f4.1)') fsamin
      write(*,'(''  Sample Interval (msec)='',f7.4)') fsamin
      fsamin=fsamin*1000.
      endif
c...delay time
      if(ntr.eq.19) then
      read(line(1:4),'(f4.1)') delayt
      write(*,'(''  Delay Time (msec)='',f7.4)') delayt
      endif
c...low cut filter
      if(ntr.eq.20) then
      read(line(1:4),'(I4)') iflc
      write(*,'(''  Low Cut Filter Hz='',I4)') iflc
      endif
c...high cut filter
      if(ntr.eq.21) then
      read(line(1:4),'(I4)') ifhc
      write(*,'(''  High Cut Filter Hz='',I4)') ifhc
      endif
c...channel information
      if(ntr.gt.25) then
      if(mod((ntr-25),8).eq.1) jchac=jchac+1
      kk=(jchac-1)*8+26
      if(ntr.eq.(kk)) read(line(1:4),'(i4)') nvsa(jchac)
      if(ntr.eq.(kk+1)) read(line(1:4),'(i4)') gidba(jchac)
      endif
c
      go to 10
 20   ntrace=nchan
c//////////////////////////end bison file read///////////////
c
      open(unit=3,file=ofile,access='sequential')
c...Y2K issue (this will work until 2050, a year I won't care about)
      if(iyear.lt.50) then
c-------assume year starts 20_iyear
      write(3,900) iflc,ifhc,iyear,idate,linenm,ihour,imin,fshdp,irec
      else
c-------assume year starts 19_iyear
      write(3,902) iflc,ifhc,iyear,idate,linenm,ihour,imin,fshdp,irec
      endif
c
c...Begin Loop, Read *.nez Topcon Survey File------------------
      jrec=0
      iexit=0
  500 continue
      jrec=jrec+1
      read(1,'(5A12)',end=600) point,ycoord,xcoord,elev,label
c
c...acquire shot coordinates
      if(label(1:2).eq.SPL) then
      nend=12
      do 42 j=3,12
      if(label(j:j).eq.blank) then
      nend=j-1
      go to 43
      endif
   42 continue
   43 continue
      itest=nend-2
      if(itest.eq.1) read(label(3:3),'(I1)') issn
      if(itest.eq.2) read(label(3:4),'(I2)') issn
      if(itest.eq.3) read(label(3:5),'(I3)') issn
      if(itest.eq.4) read(label(3:6),'(I4)') issn
      if(issn.eq.ispn) then
      read(ycoord(1:12),'(f12.0)') sy
      read(xcoord(1:12),'(f12.0)') sx
      read(elev(1:12),'(f12.0)') sz
      write(*,*) sx,sy,sz
      endif
      endif
c
c...acquire geophone coordinates
      if(label(1:2).eq.VPL) then
      nend=12
      do 44 j=3,12
      if(label(j:j).eq.blank) then
      nend=j-1
      go to 45
      endif
   44 continue
   45 continue
      itest=nend-2
      if(itest.eq.1) read(label(3:3),'(I1)') ivpn
      if(itest.eq.2) read(label(3:4),'(I2)') ivpn
      if(itest.eq.3) read(label(3:5),'(I3)') ivpn
      if(itest.eq.4) read(label(3:6),'(I4)') ivpn
      ichan=(ivpn-ivp1)/islop + 1
ccc   write(3,*) ivpn,ichan
      if(ichan.le.nchanl .and. ichan.ge.1) then
      read(ycoord(1:12),'(f12.0)') ry(ichan)
      read(xcoord(1:12),'(f12.0)') rx(ichan)
      read(elev(1:12),'(f12.0)') rz(ichan)
      chan(ichan)=ichan
      vp(ichan)=ivpn
      endif
      endif
      if(jrec.gt.ndim) then
      write(*,*) '  ABORT! dimensions'
      stop
      endif
      go to 500
c-------------------------end loop--read *.nez Survey file 
  600 continue
      nrec=jrec-1
      write(*,'(''  number of traces='',i5)') nrec
c
c...test for missing points
      do 201 j=1,nchanl
      if(chan(j).eq.0) then
      write(*,'(''  Channel = '',i5,'' Missing'')') j
      k1=j-1
      k2=j+1
      if(k1.le.nchanl .and. k1.ge.1 .and. k2.le.48 .and. k2.ge.1) then
      do 22 k=1,nchanl
      if(chan(k1).ne.0) go to 23
      k1=k1-1
   22 continue
   23 continue
      do 24 k=1,nchanl
      if(chan(k2).ne.0) go to 25
      k2=k2+1
   24 continue
   25 continue
      write(*,*) k1,j,k2
c...interpolate...but only if bracketed...
      if(k1.le.nchanl .and. k1.ge.1 .and.
     +  k2.le.nchanl .and. k2.ge.1) then
      chan(j)=j
      rx(j)=rx(k1)+(float(j-k1)/float(k2-k1))*(rx(k2)-rx(k1))
      ry(j)=ry(k1)+(float(j-k1)/float(k2-k1))*(ry(k2)-ry(k1))
      rz(j)=rz(k1)+(float(j-k1)/float(k2-k1))*(rz(k2)-rz(k1))
      fi=sign(0.5,float(vp(k2)-vp(k1)))
      vp(j)=vp(k1)+int((float(j-k1)/float(k2-k1))*(vp(k2)-vp(k1))+fi)
      else
      write(*,'(1x,70(1h-))')
      write(*,*)'  CAUTION !! Unable to Interpolate...Must Edit File'
      write(*,'('' Edit channel ='',i5)') j
      write(*,'(1x,70(1h-))')
      chan(j)=j
      endif
      else
      write(*,'(1x,70(1h-))')
      write(*,*)'  CAUTION !! Unable to Interpolate...Must Edit File'
      write(*,'('' Edit channel ='',i5)') j
      write(*,'(1x,70(1h-))')
      chan(j)=j
      endif
      endif
  201 continue
c
c...output results to *.xyz file-------------------
      sz=sz+eshift
      do 101 j=1,nchanl
      pick(j)=0.0
c...apply correction to elevations
      rz(j)=rz(j)+eshift
      ixazi=0
      ixvr=0
      if(nchanl.eq.6) then
      if(j.eq.1) then
      ixazi=0
c---Geophone/Tool Wiring Specifications
c...............use ixvr=180 for MARK Products SWC Phone
c     ixvr=180
c...............use ixvr=0   for OYO 14Hz BHG-2 Geostuff Tool
      ixvr=0
      endif
      if(j.eq.2) then
      ixazi=0
      ixvr=90
      endif
      if(j.eq.3) then
      ixazi=0
      ixvr=90
      endif
      if(j.eq.5) then
      ixazi=irazi
      ixvr=90
      endif
      if(j.eq.6) then
      ixazi=itazi
      ixvr=90
      endif
      endif
      write(3,901) chan(j),pick(j),ispn,sx,sy,sz,
     +vp(j),rx(j),ry(j),rz(j),gidba(chan(j)),isaz,isvr,ixazi,ixvr
  101 continue
c--------------------end write loop, *.xyz file
c
      write(*,*) ' '
      write(*,'('' Output in file ===> '',a)') ofile
c
c...FORMAT BLOCK
  900 format(1x,'&bhed'/5x,'lowcut=',i4,', highct=',i4,', year=20',i2.2,
     +', day=',i4.4,','/5x,' line=''',a,''', hour=',i2.2,
     +', minute=',i2.2,','/
     +5x,'sdepth=',f4.1,', uphole=0.000, phone=''VERT'', srec=',i4.3,
     +','/1x,'&end')
  902 format(1x,'&bhed'/5x,'lowcut=',i4,', highct=',i4,', year=19',i2.2,
     +', day=',i4.4,','/5x,' line=''',a,''', hour=',i2.2,
     +', minute=',i2.2,','/
     +5x,'sdepth=',f4.1,', uphole=0.000, phone=''VERT'', srec=',i4.3,
     +','/1x,'&end')
  901 format(1x,i3.3,1x,f6.4,3x,i4.3,3(1x,f10.3),3x,i4.3,3(1x,f10.3),
     +2x,i3,4(1x,i3.3))
      close(1)
      close(2)
      close(3)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | readrc: subroutine to read a bison header record           |
c |  on first call, icall=0 to open file, jrec=0               |
c |  jrec accumulates with each call to keep track of location |
c |       in the file.  nchar= number of characters in the line|
c |                                                            |
c |------------------------------------------------------------| 
c
      Subroutine readrc(iunit,icall,infil,jrec,line,nchar,iexit)
      character*1 jbyte,cr,lf,etx
      character*80 line
      character*40 infil
c
      cr=char(13)
      lf=char(10)
      etx=char(03)
      iexit=0
c
c...start of first call-------------------------------------------
      if(icall.eq.0) then
      nrecl=1
      open(iunit,file=infil,access='direct',
     +form='UNFORMATTED',recl=nrecl)
c    +form='binary',recl=nrecl)
      icall=1
      endif
      nchar=0
c
  500 jrec=jrec+1
      read(iunit,rec=jrec) jbyte
      nchar=nchar+1
c...test for end of header
      if(jbyte.eq.etx) then
      jrec=jrec+1
      read(iunit,rec=jrec) jbyte
      if(jbyte.eq.etx) then
      iexit=1
      go to 600
      endif
      endif
c...test for end of line
      if(jbyte.eq.cr) then
      jrec=jrec+1
      read(iunit,rec=jrec) jbyte
      if(jbyte.eq.lf) then
      nchar=nchar-1
      go to 600
      endif
      else
c      
      line(nchar:nchar)=jbyte
      endif
      go to 500
600   continue
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil1,infil2,linenm,fshdp,ispn,
     +nchanl,ivp1,ivp48,irec,eshift,isaz,isvr,irazi,itazi,islop)
c
      character*4       linenm
      character*40      infil1,infil2
      character*40      arg1,arg3,arg4,arg5,arg6
      character*40      arg7,arg8,arg9,arg10,arg11,arg12
      character*40      arg13,arg14                        
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '|topcon: combines Topcon survey *.nez data |'
      write(*,*) '|with BISON file data-->*.xyz file-->bhed.f|'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'topcon  topf bisf lid shdp is nch vp1 vpn ir esh isa isv ira ita'
      write(*,*) ' '
      write(*,*) 'topf     =file name of topcon .nez file  '
      write(*,*) 'bisf     =file name of Bison file with data'
      write(*,*) 'lid      =line ID   '
      write(*,*) 'shdp     =shot depth '
      write(*,*) 'is       =shot location number '
      write(*,*) 'nch      =number of channels (<48) '
      write(*,*) 'vp1      =geophone location number channel 1 '
      write(*,*) 'vpn      =geophone location number channel nchanl '
      write(*,*) 'ir       =shot record number '
      write(*,*) 'esh      =elevation adjustment to be added '
      write(*,*) 'isa      =source polarization azimuth (deg.) '
      write(*,*) 'isv      =source polarization vertical (deg.) '
      write(*,*) 'ira      =reference polarization R-axis (deg.) '
      write(*,*) 'ita      =reference polarization T-axis (deg.) '
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  Enter Topcon File Name'
      read(*,'(a13)') infil1
      else
      call getarg(1,infil1)
      endif
      call namechk(12,infil1)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*) '  Enter Bison File Name'
      read(*,'(a13)') infil2
      else
      call getarg(2,infil2)
      endif
      call namechk(12,infil2)
c
c
c...input shot and receiver info
      if(nargsx.lt.(4+karg)) then
      write(*,*) ' Enter Line Name (4 Char.)'
      read(*,'(a)') linenm
      else
      call getarg(3,arg3)
      read(arg3,'(a)') linenm
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*) ' Enter Shot Depth'
      read(*,*) fshdp
      else
      call getarg(4,arg4)
      read(arg4,'(f10.4)') fshdp
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,*) ' Enter Shot SP Number (ie. 1)'
      read(*,*) ispn
      else
      call getarg(5,arg5)
      read(arg5,'(i3)') ispn
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,*) ' Enter Number of Channels/shot record'
      read(*,*) nchanl
      else
      call getarg(6,arg6)
      read(arg6,'(i4)') nchanl
      endif
c
      if(nargsx.lt.(8+karg)) then
      write(*,*) ' Enter Receiver VP Number for Channel 1 (ie. 1)'
      read(*,*) ivp1
      else
      call getarg(7,arg7)
      read(arg7,'(i4)') ivp1
      endif
c
      if(nargsx.lt.(9+karg)) then
      write(*,*) ' Enter Receiver VP Number for Channel nchnl (ie.48)'
      read(*,*) ivp48
      else
      call getarg(8,arg8)
      read(arg8,'(i4)') ivp48
      endif
c
      if(nargsx.lt.(10+karg)) then
      write(*,*) ' Enter sequential shot record number'
      read(*,*) irec
      else
      call getarg(9,arg9)
      read(arg9,'(i4)') irec
      endif
c
      if(nargsx.lt.(11+karg)) then
      write(*,*) ' Enter Elevation Shift (added to topcon elev)'
      read(*,*) eshift
      else
      call getarg(10,arg10)
      read(arg10,'(f10.4)') eshift
      endif
      ii=ivp48-ivp1
      ii=ii/iabs(ii)
      islop=(ivp48-ivp1+ii)/nchanl
      if(iabs(islop).ne.1) then
      write(*,*) '  ABORT!!! more than nchanl channels specified'
      stop
      endif
c
      if(nargsx.lt.(12+karg)) then
      write(*,*) ' Source Polarization: Enter Azimuth (deg.) '
      read(*,*) isaz
      else
      call getarg(11,arg11)
      read(arg11,'(i5)') isaz
      endif
c
      if(nargsx.lt.(13+karg)) then
      write(*,*) ' Source Polarization: Enter Vertical (deg.) '
      read(*,*) isvr
      else
      call getarg(12,arg12)
      read(arg12,'(i5)') isvr
      endif
c
      if(nargsx.lt.(14+karg)) then
      write(*,*) ' Reference Polarization: Enter R-azimuth (deg.) '
      read(*,*) irazi
      else
      call getarg(13,arg13)
      read(arg13,'(i5)') irazi
      endif
c
      if(nargsx.lt.(15+karg)) then
      write(*,*) ' Reference Polarization: Enter T-azimuth (deg.) '
      read(*,*) itazi
      else
      call getarg(14,arg14)
      read(arg14,'(i5)') itazi
      endif
      return
      end
