// |$Id: fnorm.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $         |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: fnorm.c                                     |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: normalize a trace (L2 or input factor)            |
   |                                                            |
   |INPUTS:                                                     |
   |  s1      =input trace array                                |
   |  npts    =number samples                                   |
   |  Pointers:                                                 |
   | *fscal 0.=use L2 norm                                      |
   |          =use 1/fscal as scale factor                      |
   |                                                            |
   |OUTPUTS: pointers                                           |
   |  s2     =output signal                                     |
   |                                                            |
   |  s2(j)=s1(j)/fscal         <===formula                     |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <math.h>

      int  fnorm(int npts, float *fscal, float s1[], float s2[])
     {

//...declare variables
      int               j;
      float             sum;

//...parameter check
      if(npts<1)
     {puts("ABORT: function fnorm");
      puts("   invalid parameter");
      return(1);
     }

//...determine scale factor
      if(*fscal==0.)
     {
      sum=0.;
      for (j=0;j<npts;j++) sum=sum+s1[j]*s1[j];
      if(sum!=0.) *fscal=sqrt(sum);
      else
      *fscal=.0;
     }  // endif

//...apply factor
      if(*fscal!=0.)
      for (j=1;j<npts;j++) s2[j]=s1[j]/(*fscal);
      else
      for (j=1;j<npts;j++) s2[j]=s1[j];
      return(0);
     } //  end fnorm
