// |$Id: lsqufit.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: lsqufit.c                                   |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   | MODIFIED: 2 June 2002  Convert internal computations to    |
   |                        double precision                    |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   | purpose: least squares fit                                 |
   |          isw1   0=Linear 1=Log_e for y-variable            |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
	#include <math.h>
	#include <stdio.h>

      void lsqufit(float x[], float y[], int npts, int isw1,
         float *slope, float *intrcpt, float *lsqe, float *r2)
     {
      int            i;
      double         fn,
                     err,
                     sum,
                     sumx,
                     sumy,
                     sumy2,
                     sumx2,
                     sumxy;

//...compute sums
      sumx=0.;
      sumy=0.;
      sumx2=0.;
      sumxy=0.;
      sumy2=0.;

      for(i=0;i<npts;i++)
     {
      sumx=sumx+(double)x[i];
      sumx2=sumx2+x[i]*(double)x[i];
        if(isw1==0)
       {
        sumy=sumy+(double)y[i];
        sumy2=sumy2+(double)y[i]*(double)y[i];
        sumxy=sumxy+(double)x[i]*(double)y[i];
       }
        else
       {
        sumy=sumy+ log((double)y[i]);
        sumy2=sumy2+ log((double)y[i])*(float)log((double)y[i]);
        sumxy=sumxy+(double)x[i]*(log((double)y[i]));
       }  //endif
     }  //endfor

//...compute slope and intercept
      fn=(double)npts;
      *slope=(fn*sumxy-sumy*sumx) / (fn*sumx2-sumx*sumx);
      *intrcpt=(sumx2*sumy-sumx*sumxy) / (fn*sumx2-sumx*sumx);
//...compute least squares error
      sum=0.0;
      for(i=0;i<npts;i++)
     {
        if(isw1==0)
       {
        err=x[i]*(*slope)+(*intrcpt)-y[i];
       }
        else
       {
        err=exp(x[i]*(*slope)+(*intrcpt))-y[i];
       }  //endif

      sum=sum+err*err;

     } //endfor
//...leastsquare error
      *lsqe=sqrt(sum/fn);

//...coef of determination
      *r2=(sumxy-(sumx*sumy)/fn)*(sumxy-(sumx*sumy)/fn)/
       ((sumx2-sumx*sumx/fn)*(sumy2-sumy*sumy/fn));

     }  // end lsqufit
