c |$Id: bsegin.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c |------------------------------------------------------------|
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bsegin.f                                     |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | bsegin: inputs a trace in bsegy format with headers        |
c |                                                            |
c |  traces=vector with data                                   |
c |  npts= number of values in traces to be output             |
c |  jrec= record number for this trace                        |
c |  infil =   input file name (opened on first call of segin) |
c |  fsamin=sample rate of data                                |
c |  iexit= switch to detect end of file                       |
c |   note: main program specifies icall                       |
c |   set icall = 0 on first call                              |
c |------------------------------------------------------------|
c
      SUBROUTINE BSEGIN(iunit,ndim,
     +icall,npts,traces,fsamin,infil,jrec,iexit)
      include "../Fort/include/bsegy.inc"
      real*4       traces(*)
      character*40 infil
c
      common /header/ hd
c
c...set exit switch
      iexit=0
c
c...start of first call-------------------------------------------
      if(icall.eq.0) then
c
c//////////////////////////////////////////////////////////////////
c...non segy convention for large sample rates (integer*2 isi) ////
c//// segy convention is for seismic data sample rates in      ////
c//// microseconds.  problem is that an integer*2 variable was ////
c//// chosen for this variable.  fix up here is to use a seldom////
c//// needed variable, duse, as a switch to scale the sample   ////
c//// rate to other ''units''.                                 ////
c//////////////////////////////////////////////////////////////////
c
c  find the number of data points in SEG-Y file, and sample rate
c
      open(iunit,file=infil,access='direct',form='unformatted',recl=240)
c
      read(iunit,rec=1,err=600)
     +(hd(k),k=1,240)
c
      npts=length
c
      if(npts.gt.ndim) then
      write(*,*)'  ABORT!!!!---dimensions exceeded'
      write(*,'('' Maximum Trace Length = '',i5)') ndim
      write(*,'('' Current Trace Length = '',i5)') npts
      write(*,*)'  Use BEDT to resample data OR'
      write(*,*)'    increase program dimensions'
      stop
      endif
c
c
c...non-segy convention for isi>32768 (large sample rates)
c...convert to seconds from microseconds
      if( duse .lt. 2 ) then
      fsamin = float(isi) / 1.0e+06
      else
      fsamin = float(isi) / 1.0e+03
      endif
c
      close(iunit)
c
c open file with the correct record length
      nlen=240+4*length
      open (iunit,file=infil,access='direct',form='unformatted',
     +recl=nlen)
      icall=1
      endif
c---------------------------------end of first call
c
c...input trace
      read(iunit,rec=jrec,err=600)
     +(hd(k),k=1,240),
     +(traces(j),j=1,length)
c
      go to 500
600   iexit=1
500   continue
      return
      end
