c |$Id: cfilt.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: cfilt.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | cfilt:  cosine taper band pass filters that overlap        |
c |         giving a total unity pass band from freq1 to       |
c |         freq1 + nfil*deltf                                 |
c |                                                            |
c | filters returned in fil(npf,nfil) (logical dimensions)     |
c |                                                            |
c |------------------------------------------------------------|
      subroutine cfilt(fil,nfil,npf,deltf,freq1,hw,dt,env,im)
      real*4 fil(im,*),env(npf)
c
c...make npf an odd number
      npfx=(npf/2)*2 + 1
      if(npfx.gt.npf) npfx=npfx-2
      pi=3.1415926
      pi2=pi*2.0
c
c...constant bandwidth filters
      tn=1.0/hw
      nph=npfx/2+1
      nphm1=nph-1
      nphm2=nph-2
      pi2rn=pi2/(npfx-1)
      con=pi/(tn*tn)
c
c...compute cosine taper envelope and store in env
      do 10 i=1,nph
      ii=i+nphm2
      arg=ii*pi2rn
      env(i)=(0.5-0.5*cos(arg))
   10 continue
c
c...modulation principle (multiply envelope by cosine at
c   center frequency, fc)
      do 20 ifil=1,nfil
      fc=(ifil-1)*deltf+freq1
c     write(*,'(5x,''center freq='',f10.5,'' filter #='',i5)') fc,ifil
      fil(nph,ifil)=env(1)
      pi2fdt=pi2*fc*dt
      do 30 ip=1,nphm1
      i1=nph+ip
      i2=nph-ip
      arg=pi2fdt*ip
      coef=cos(arg)*env(ip+1)
      fil(i1,ifil)=coef
      fil(i2,ifil)=coef
   30 continue
   20 continue
      return
      end
