c |$Id: flist.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: flist.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | flist: lists a complex trace                               |
c |                                                            |
c | samin = sample rate                                        |
c | tzero = zero reference time for phase                      |
c | ilin  = 1 for linear scale  0= decibel scale               |
c |                                                            |
c | Prints to io6 which is hardwired to unit 10.               |
c | (HINT: subroutines like lstprm.f typically write to unit 11|
c | so, a good idea is to set lstprm output to unit 10 if you  |
c | want to have this subroutine write to the listing file).   |
c --------------------------------------------------------------
c
      subroutine flist(npts,x,title,samin,tzero,xr,xi,amp,phz,ilin)
      real*4 xr(*),xi(*),amp(*),phz(*)
      complex x(*)
      character*1 star,blank,ya(23),yp(23),dot
      character*70 title
      EXTERNAL cshift
      write(star,'(''*'')')
      write(dot,'(''.'')')
      write(blank,'('' '')')
      if(npts.le.0) return
      write(10,'(''  ...samin='',e10.4)') samin
c
      pi=3.14159265358979323846264338327950
c
c...rotate phase
      dt=-tzero
      call cshift(npts,x,dt,samin)
c
c...demultiplex to real and imag parts
      call demult(npts,x,xr,xi)
c
c...convert to polar coord.
      call polar(npts,xr,xi,amp,phz)
c
c...find max and minimum values in trace
      call xmax(npts,amp,fmaxa,fmina)
      call xmax(npts,phz,fmaxp,fminp)
      fmaxp=(180./pi)*fmaxp
      fminp=(180./pi)*fminp
      do 5 j=1,23
      ya(j)=blank
    5 yp(j)=blank
      rangdb=60.
      fmindb=-60.
      rangep=360.
      pmin=-180.
      pmax=+180.
c
c...test if linear plot
      if(ilin.eq.1) then
      rangdb=10.0
      fmindb=0.0
      endif
      yp(12)=dot
      io6=10
      lc=0
      jmax=npts/2
c
c...write max and min
      if(ilin.eq.0) then
      write(io6,900) char(12),fmaxa,fmina,fmaxp,fminp,tzero
      else
      write(io6,904) char(12),fmaxa,fmina,fmaxp,fminp,tzero
      endif
c
      do 10 j=1,jmax
      if(lc.lt.50) go to 7
      write(io6,902) title
  902 format(26x,'|',23('.'),'|',23('.'),'|'/5x,a)
      lc=0
c
      if(ilin.eq.0) then
      write(io6,900) char(12),fmaxa,fmina,fmaxp,fminp,tzero
      else
      write(io6,904) char(12),fmaxa,fmina,fmaxp,fminp,tzero
      endif
  900 format(a/2x,'maxa=',e9.3,' mina=',e9.3,
     +' maxp=',e9.3,' minp=',e9.3,' tzero=',f6.3/
     +3x,'j',3x,'freq',4x,'amp',4x,'phz',1x,5('.'),
     +'log scale',8('.'),'-180',10('.'),'0',9('.'),
     +'+180')
  904 format(a/2x,'maxa=',e9.3,' mina=',e9.3,
     +' maxp=',e9.3,' minp=',e9.3,' tzero=',f6.3/
     +3x,'j',3x,'freq',4x,'amp',4x,'phz',1x,5('.'),
     +'linear scale',5('.'),'-180',10('.'),'0',9('.'),
     +'+180')
    7 continue
c
c...plot a point
      ampl=amp(j)/fmaxa
c...test if linear plot
      if(ilin.eq.0) then
      if(ampl.lt.1.e-3) ampl=1.e-3
      ampl=20.*alog10(ampl)
      else
      ampl=ampl*10.0
      endif
c
      fka=(ampl-fmindb)/rangdb
      ka=int(fka*22.+1)
      do 20 jp=1,ka
   20 ya(jp)=star
      phaz=(180./pi)*phz(j)
      fkp=(phaz-pmin)/rangep
      kp=int(fkp*22.+1)
      yp(kp)=star
      freq=(j-1)/(npts*samin)
      if(freq.lt.1.) go to 44
      write(io6,901) j,freq,ampl,phaz,(ya(jk),jk=1,23),
     +(yp(jl),jl=1,23)
      go to 45
   44 write(io6,903) j,freq,ampl,phaz,(ya(jk),jk=1,23),
     +(yp(jl),jl=1,23)
   45 continue
  901 format(1x,i3,1x,f6.1,1x,f6.1,1x,f6.1,' |',23a,
     +'|',23a,'|')
  903 format(1x,i3,1x,f6.5,1x,f6.1,1x,f6.1,' |',23a,
     +'|',23a,'|')
      lc=lc+1
c
c...restore plot line
      do 30 jp=1,ka
   30 ya(jp)=blank
      yp(kp)=blank
      yp(12)=dot
   10 continue
      write(io6,902) title
      call cshift(npts,x,tzero,samin)
      return
      end
