c |$Id: shape.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: E. A. Robinson (1967, p.75)                        |
c | PROGRAM NAME: shape.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c | Copyright (c) 1967 by Holden-Day Inc. 500 Sansome St.,     |
c | San Francisco, all rights reserved                         |
c |                                                            |
c | Holden-Day Inc., Liquidation Trust, 1259 S.W. 14th Street  |
c | Boca Raton, FL 33486 Phone: (561)750-9229 Fax: 561.394.6809|
c |                                                            |
c |    This subroutine is included for distribution with Basic |
c | Seismic Utilities (BSU) by joint permission of the author, |
c | Enders Robinson, and Holden-Day Inc. Liquidation Trust,    |
c | Frederick H. Murphy, Trustee.  Being part of the BSU       |
c | package, it is governed by the terms of the GPL license,   |
c | and is distributed in the hope that it will be useful, but |
c | WITHOUT ANY WARRANTY; without even the implied warranty    |
c | of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    |
c | See the GNU General Public License for more details.       |
c | Paul Michaels, February 2002 <pm@cgiss.boisestate.edu>     |
c |                                                            |
c |------------------------------------------------------------|
c | Modification:  Added "wht" term to stabalize algorithm     |
c | in the event that spectral minima are present in the data. |
c | (new argument in call, also new line 5, space(1)=...       |
c | adding bias to zero lag of autocorrelation.) PM.           |
c |------------------------------------------------------------|
c |Minor modifications, Fortran IV to Fortran 77 by P. Michaels|
c |------------------------------------------------------------|
c |                                                            |
c | subroutines:  cross, eureka, dot, fold, shape              |
c |                                                            |
c |   b = input vector, length lb                              |
c |   d = desired output vector, length ld                     |
c |   a = shaping filter, length la                            |
c |   space = work space vector                                |
c |   c = actual output of filter, length lc=la+lb-1           |
c |   ase = average squared error between actual and desired   |
c |   wht = percent whitening (bias added to zero lag autocor) |
c |         for stability                                      |
c --------------------------------------------------------------
c
      subroutine shape(lb,b,ld,d,la,a,lc,c,ase,space,wht)
      implicit real*4 (a-h,o-z)
c     real*4 b(lb),d(ld),a(la),c(lc),space(3*la)
      real*4 b(*),d(*),a(*),c(*),space(*)
      call cross(lb,b,lb,b,la,space)
      space(1)=wht*space(1)+space(1)
      call cross(ld,d,lb,b,la,space(la+1))
      call eureka(la,space,space(la+1),a,space(2*la+1))
      call dot(ld,d,d,dd)
      call dot(la,a,space(la+1),ag)
      ase=(dd-ag)/dd
      call fold(la,a,lb,b,lc,c)
      return
      end
